package com.yunzhi.training.controller;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.yunzhi.training.common.*;
import com.yunzhi.training.entity.SysUser;
import com.yunzhi.training.entity.TaPerson;
import com.yunzhi.training.entity.TaStudent;
import com.yunzhi.training.service.*;
import com.yunzhi.training.vo.ChangePassword;
import com.yunzhi.training.vo.AdminLoginParam;
import com.yunzhi.training.vo.TokenParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

@Api(tags = "登入/登出")
@RestController
public class LoginController extends BaseController {

    @Autowired
    SMSCaptcha smsCaptcha;

    @Autowired
    ISysUserService iSysUserService;

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    ITaStudentService iTaStudentService;

    @Autowired
    public ITaSchoolTermService iTaSchoolTermService;

    @Autowired
    public ITaSchoolClassService iTaSchoolClassService;

    @Value("${yz.mp.openidAPI}")
    String openidAPI;

    @Autowired
    HttpUtils httpUtils;

    @PostMapping("/admin/login")
    @ApiOperation(value="登录", notes = "登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean login(@ApiParam("登录参数") @RequestBody AdminLoginParam loginParam) throws Exception {
        if (null == loginParam) {
            return ResponseBean.error("账户或密码不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        String userName = loginParam.getUserName();
        String password = loginParam.getPassword();

        if (StringUtils.isEmpty(userName) || StringUtils.isEmpty(password)) {
            return ResponseBean.error("账户或密码不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        SysUser sysUser = iSysUserService.getByLoginName(userName);
        if (null == sysUser) {
            return ResponseBean.error("账户或密码不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (!checkPassword(password, sysUser.getPassword(), sysUser.getUserId())) {
            return ResponseBean.error("账户或密码不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (Constants.STATUS_NORMAL != sysUser.getState()) {
            return ResponseBean.error("用户状态不正确, 请联系管理人员", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 生成 token
        TokenParam tokenParam = new TokenParam()
                .setUserId(sysUser.getUserId())
                .setOrgId("0");

        Map<String, Object> tokenMap = tokenParam.toMap();
        String token = JWTUtils.encode(tokenMap);

        Map<String, Object> res = new HashMap<String, Object>() {{
            put("user", sysUser);
            put("token", token);
        }};
        return ResponseBean.success(res);
    }


    @PutMapping("/admin/change-password")
    @ApiOperation(value="修改密码", notes = "修改密码", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean changePassword(@ApiParam("修改密码参数") @RequestBody ChangePassword param,
                                       HttpServletRequest request) throws Exception {
        if (StringUtils.isEmpty(param.getOriginPassword()) || StringUtils.isEmpty(param.getNewPassword())) {
            return ResponseBean.error("原始密码或新密码不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        SysUser currentUser = getCurrentUser(request);

        if (!checkPassword(param.getOriginPassword(), currentUser.getPassword(), currentUser.getUserId())) {
            return ResponseBean.error("原始密码不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        currentUser.setPassword(EncryptUtils.md5(param.getNewPassword(), currentUser.getUserId()));
        iSysUserService.updateById(currentUser);

        return ResponseBean.success("密码修改成功");
    }

    @ApiParam("客户端登录")
    @PostMapping("/wx/login")
    public ResponseBean clientLogin(@ApiParam("code") @RequestParam String code) throws Exception {
        if (StringUtils.isEmpty(code)) {
            return ResponseBean.error("登录参数不存在", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        String openID = null;
        if (!code.equals("123")) {
            String body = httpUtils.get(String.format(openidAPI, code));
            if (StringUtils.isEmpty(body)) {
                return ResponseBean.error("登录系统失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
            }

            JSONObject jsonObject = JSONUtil.parseObj(body);
            if (jsonObject.getInt("code") != ResponseBean.CODE_SUCCESS) {
                return ResponseBean.error(jsonObject.getStr("message"), ResponseBean.ERROR_UNAVAILABLE);
            }

            openID = jsonObject.getStr("data");
        } else {
            openID = "123";
        }

        TaPerson taPerson = iTaPersonService.getByOpenID(openID);
        if (null == taPerson) {
            taPerson = new TaPerson();
            taPerson.setOpenid(openID);
            taPerson.setState(Constants.STATUS_NORMAL);
            if (!iTaPersonService.save(taPerson)) {
                return ResponseBean.error("登录系统失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
            }
        } else {
            if (Constants.STATUS_NORMAL != taPerson.getState()) {
                return ResponseBean.error("您的账户异常, 请联系管理人员", ResponseBean.ERROR_UNAVAILABLE);
            }
        }

        // 先找关联学生
        TaStudent taStudent = iTaStudentService.getExistBy("person_id", taPerson.getPersonId(), false, true);
        if (taStudent == null) {
            // 未找到再依据手机号查找
            if (!StringUtils.isEmpty(taPerson.getPhone())) {
                taStudent = iTaStudentService.getExistBy("phone", taPerson.getPhone(), false, true);
                if (taStudent != null && StringUtils.isEmpty(taStudent.getPersonId())) {
                    taStudent.setPersonId(taPerson.getPersonId());
                    try {
                        iTaStudentService.updateById(taStudent);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        if (null != taStudent) {
            if (!StringUtils.isEmpty(taStudent.getTermId())) {
                taStudent.setTermName(iTaSchoolTermService.getById(taStudent.getTermId()).getName());
            }
            if (!StringUtils.isEmpty(taStudent.getClassId())) {
                taStudent.setClassName(iTaSchoolClassService.getById(taStudent.getClassId()).getName());
            }
        }

        // 生成 token
        TokenParam tokenParam = new TokenParam()
                .setUserId(taPerson.getPersonId())
                .setOrgId("0");

        Map<String, Object> tokenMap = tokenParam.toMap();
        String token = JWTUtils.encode(tokenMap);

        Map<String, Object> res = new HashMap<>();
        res.put("person", taPerson);
        res.put("student", taStudent);
        res.put("token", token);

        return ResponseBean.success(res);
    }

    private boolean checkPassword(String src, String targ, String salt) {
        return EncryptUtils.md5(src, salt).equals(targ);
    }
}
